package io.hmit.modules.serviceOrder.controller;

import io.hmit.common.annotation.LogOperation;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.ExcelUtils;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.serviceOrder.dto.PensionServiceTypeDTO;
import io.hmit.modules.serviceOrder.excel.PensionServiceTypeExcel;
import io.hmit.modules.serviceOrder.service.PensionServiceTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 服务种类表
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-01-26
 */
@RestController
@RequestMapping("serviceOrder/pensionservicetype")
@Api(tags="服务种类表")
public class PensionServiceTypeController {
    @Autowired
    private PensionServiceTypeService pensionServiceTypeService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
    @RequiresPermissions("serviceOrder:pensionservicetype:page")
    public Result<PageData<PensionServiceTypeDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<PensionServiceTypeDTO> page = pensionServiceTypeService.page(params);

        return new Result<PageData<PensionServiceTypeDTO>>().ok(page);
    }

    @GetMapping("getServiceTypeList")
    @ApiOperation("查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "keyword", value = "查询字段，", paramType = "query", dataType="String")
    })

    public Result<List<PensionServiceTypeDTO>> getServiceTypeList(@ApiIgnore @RequestParam Map<String, Object> params){
        List<PensionServiceTypeDTO> pensionServiceTypeDTOList = pensionServiceTypeService.list(params);

        return new Result<List<PensionServiceTypeDTO>>().ok(pensionServiceTypeDTOList);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @RequiresPermissions("serviceOrder:pensionservicetype:info")
    public Result<PensionServiceTypeDTO> get(@PathVariable("id") Long id){
        PensionServiceTypeDTO data = pensionServiceTypeService.get(id);

        return new Result<PensionServiceTypeDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @RequiresPermissions("serviceOrder:pensionservicetype:save")
    public Result save(@RequestBody PensionServiceTypeDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        pensionServiceTypeService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @RequiresPermissions("serviceOrder:pensionservicetype:update")
    public Result update(@RequestBody PensionServiceTypeDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        pensionServiceTypeService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @RequiresPermissions("serviceOrder:pensionservicetype:delete")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        pensionServiceTypeService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @RequiresPermissions("serviceOrder:pensionservicetype:export")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<PensionServiceTypeDTO> list = pensionServiceTypeService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, PensionServiceTypeExcel.class);
    }

}
