package io.hmit.modules.serviceOrder.controller;

import io.hmit.annotation.Login;
import io.hmit.annotation.LoginUser;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.entity.UserEntity;
import io.hmit.modules.serviceOrder.dto.PensionDocumentDTO;
import io.hmit.modules.serviceOrder.dto.PensionOrderDocumentDTO;
import io.hmit.modules.serviceOrder.entity.PensionDocumentEntity;
import io.hmit.modules.serviceOrder.service.PensionDocumentService;
import io.hmit.modules.serviceOrder.service.PensionOrderDocumentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;


/**
 * 订单及相关图片附件关联表
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-03-11
 */
@RestController
@RequestMapping("serviceOrder/pensionorderdocument")
@Api(tags="订单及相关图片附件关联表")
public class PensionOrderDocumentController {
    @Autowired
    private PensionOrderDocumentService pensionOrderDocumentService;

    @Autowired
    private PensionDocumentService pensionDocumentService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
    public Result<PageData<PensionOrderDocumentDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<PensionOrderDocumentDTO> page = pensionOrderDocumentService.page(params);

        return new Result<PageData<PensionOrderDocumentDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    public Result<PensionOrderDocumentDTO> get(@PathVariable("id") Long id){
        PensionOrderDocumentDTO data = pensionOrderDocumentService.get(id);

        return new Result<PensionOrderDocumentDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    public Result save(@RequestBody PensionOrderDocumentDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        pensionOrderDocumentService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    public Result update(@RequestBody PensionOrderDocumentDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        pensionOrderDocumentService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        pensionOrderDocumentService.delete(ids);

        return new Result();
    }

    /**
     * 上传文件
     */
    @PostMapping("/uploadFile")
    @ApiOperation("附件上传")
    public Result<Map<String, Object>> uploadFile(@RequestParam("file") MultipartFile file){
        Map<String, Object> fileMap = pensionDocumentService.uploadFile(file);

//        PensionOrderDocumentDTO dto =  new PensionOrderDocumentDTO().assemblePensionOrder(fileMap,orderId,roleId);
/*
        if (null != fileMap) {
            PensionOrderDocumentDTO dto = new PensionOrderDocumentDTO();
            dto.setCreateDate(new Date());
            dto.setOrderId(orderId);
            dto.setDocId(Long.parseLong(fileMap.get("docId").toString()));
            dto.setFileName(fileMap.get("fileName").toString());
            dto.setFilePath(fileMap.get("filePath").toString());
            pensionOrderDocumentService.save(dto);
        } else {
            return null;
        }
*/
        return new Result<Map<String, Object>>().ok(fileMap);
    }
}
