package io.hmit.modules.serviceOrder.controller;


import com.baomidou.mybatisplus.extension.api.R;
import io.hmit.annotation.Login;
import io.hmit.annotation.LoginUser;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.entity.UserEntity;
import io.hmit.modules.serviceOrder.dto.PensionServiceTypeDTO;
import io.hmit.modules.serviceOrder.service.PensionServiceTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 服务种类表
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-01-26
 */
@RestController
@RequestMapping("serviceOrder/pensionservicetype")
@Api(tags="服务种类表")
public class PensionServiceTypeController {
    @Autowired
    private PensionServiceTypeService pensionServiceTypeService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
    public Result<PageData<PensionServiceTypeDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<PensionServiceTypeDTO> page = pensionServiceTypeService.page(params);

        return new Result<PageData<PensionServiceTypeDTO>>().ok(page);
    }

    @GetMapping("list")
    @ApiOperation("根据用户角色获取服务列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roleId", value = "用户角色Id", paramType = "query", dataType="int") ,
            @ApiImplicitParam(name = "isFree", value = "是否免费：1 是， 2 否", paramType = "query", dataType="String")
    })
    public Result<List<PensionServiceTypeDTO>> getServiceList( @ApiIgnore @RequestParam Map<String, Object> params){

        List<PensionServiceTypeDTO> list = pensionServiceTypeService.getServiceListByRole(params);

        return new Result<List<PensionServiceTypeDTO>>().ok(list);
    }

    @GetMapping("getAllList")
    @ApiOperation("获取所有大类")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roleId", value = "用户角色Id", paramType = "query", dataType="int")
    })
    public Result<List<PensionServiceTypeDTO>> getALlList( @ApiIgnore @RequestParam Map<String, Object> params){

        List<PensionServiceTypeDTO> list = pensionServiceTypeService.getALLListByRole(params);

        return new Result<List<PensionServiceTypeDTO>>().ok(list);
    }


    @Login
    @GetMapping("{id}")
    @ApiOperation("信息")
    public Result<PensionServiceTypeDTO> get(@PathVariable("id") Long id){
        PensionServiceTypeDTO data = pensionServiceTypeService.get(id);

        return new Result<PensionServiceTypeDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    public Result save(@RequestBody PensionServiceTypeDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        pensionServiceTypeService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    public Result update(@RequestBody PensionServiceTypeDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        pensionServiceTypeService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        pensionServiceTypeService.delete(ids);

        return new Result();
    }


}
